/* NMEA Parser example, that decode data stream from GPS receiver

   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/

#include <stdio.h>
#include <stdlib.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_log.h"
#include "nmea_parser.h"
#include "driver/gpio.h"
#include "string.h"
#include "driver/timer.h"
#include "esp_littlefs.h"


static const char *TAG = "gps_demo";
struct stat st;
esp_vfs_littlefs_conf_t conf = {
    .base_path = "/littlefs",
    .partition_label = "storage",
    .format_if_mount_failed = true,
    .dont_mount = false,
};

SemaphoreHandle_t memory_mutex;

#define CPU_STATUS                  GPIO_NUM_40
#define CPU_STATUS_ON               gpio_set_level(CPU_STATUS,1)
#define CPU_STATUS_OFF              gpio_set_level(CPU_STATUS,0)

#define TIMER_DIVIDER         (80)  //  Hardware timer clock divider
#define TIMER_SCALE           (TIMER_BASE_CLK / TIMER_DIVIDER)  // convert counter value to seconds
#define TIME_ZONE (+5)   //Beijing Time
// #define TIME_ZONE_min (+30)   //Beijing Time
#define YEAR_BASE (2000) //date in GPS starts from 2000

char WriteDataToFS(char*file_name, int pos, char*pBuffer, int write_size)
{
    // printf("write FILE: %s, pos=%d, size= %d\n",file_name,pos,write_size);
    esp_err_t ret = esp_vfs_littlefs_register(&conf);
    if (ret != ESP_OK)
    {
        printf("Failed to get LittleFS partition information (%s)\n", esp_err_to_name(ret));
    }

    FILE *f;
    if (stat(file_name, &st) == 0)     
    {
        f = fopen(file_name, "r+");
    }
    else
    {
        f = fopen(file_name, "w");
    }
  
    if(f == NULL)
    {
        fclose(f);
        esp_vfs_littlefs_unregister(conf.partition_label);
        printf("Failed to open file for writing\n");
        return 0;
    }
  
    int seek_result = fseek(f, pos, SEEK_SET);
    if (seek_result < 0)
    {
        fclose(f);
        esp_vfs_littlefs_unregister(conf.partition_label);
        printf("Failed to seek file for writing\n");
        return 0;
    }
    size_t fwrite_status = fwrite((char*)pBuffer, 1, (size_t)write_size, f);
    if(fwrite_status<=0)//if(fwrite_status != 0)
    {
        printf("written_len:%d\n", fwrite_status);
    }
    fclose(f);
    esp_vfs_littlefs_unregister(conf.partition_label);
    return(1);
}

char ATL_ReadDataFromFS(char*file_name, int pos, char*pBuffer, int read_size)
{
    // printf("Read FILE: %s, pos=%d, size= %d\n",file_name,pos,read_size);
    esp_err_t ret = esp_vfs_littlefs_register(&conf);
    if (ret != ESP_OK)
    {
        printf("Failed to get LittleFS partition information (%s)\n", esp_err_to_name(ret));return(0);
    }

    FILE *f;
    f = fopen(file_name, "r");
    if (f == NULL)
    {
        fclose(f);
        esp_vfs_littlefs_unregister(conf.partition_label);
        printf("Failed to open file for reading %s\n",file_name);
        return 0;
    }
    int seek_result = fseek(f, pos, SEEK_SET);
    if (seek_result < 0)
    {
        fclose(f);
        esp_vfs_littlefs_unregister(conf.partition_label);
        printf("Failed to seek file for reading\n");
        return 0;
    }        
    size_t fread_result = fread((char*)pBuffer, 1, (size_t)read_size, f);
    if(fread_result != read_size)
    {
        fclose(f);
        esp_vfs_littlefs_unregister(conf.partition_label);
        printf("Failed to read data from  %s\n", file_name);
        return 0;
    }
    fclose(f);
    esp_vfs_littlefs_unregister(conf.partition_label);
    return(1);
}

char ATL_DeleteFile(char* file)
{
    esp_err_t ret = esp_vfs_littlefs_register(&conf);
    if (ret != ESP_OK)
    {
        return(0);
    }
    if(unlink(file) == 0)
    {
        esp_vfs_littlefs_unregister(conf.partition_label);
        return(1);
    }    
    esp_vfs_littlefs_unregister(conf.partition_label);
    return(0);
}

char ATL_mutex_lock()
{
    if(xSemaphoreTake(memory_mutex,(TickType_t)3000/portTICK_PERIOD_MS) == pdPASS)
    {
        return 1;
    }
    return 0;
}

char ATL_mutex_unlock()
{
    if(xSemaphoreGive(memory_mutex))
    {
        return 1;
    }
    return 0;
}

void write_data()
{
    ATL_mutex_lock();
    char write_buff[1024]={0};
    memset(write_buff,0X33, 1024);	
	WriteDataToFS((char*)"/littlefs/rd_wr.txt",0,(char*)&write_buff,1024);
    ATL_mutex_unlock();
}

/**
 * @brief GPS Event Handler
 *
 * @param event_handler_arg handler specific arguments
 * @param event_base event base, here is fixed to ESP_NMEA_EVENT
 * @param event_id event id
 * @param event_data event specific arguments
 */
static void gps_event_handler(void *event_handler_arg, esp_event_base_t event_base, int32_t event_id, void *event_data)
{
    gps_t *gps = NULL;
    switch (event_id) {
    case GPS_UPDATE:
        gps = (gps_t *)event_data;
        /* print information parsed from GPS statements */
        ESP_LOGI(TAG, "%d/%d/%d %d:%d:%d => \r\n"
                 "\t\t\t\t\t\tlatitude   = %.05f°N\r\n"
                 "\t\t\t\t\t\tlongitude = %.05f°E\r\n"
                 "\t\t\t\t\t\taltitude   = %.02fm\r\n"
                 "\t\t\t\t\t\tspeed      = %fm/s",
                 gps->date.year + YEAR_BASE, gps->date.month, gps->date.day,
                 gps->tim.hour + TIME_ZONE, gps->tim.minute, gps->tim.second,
                 gps->latitude, gps->longitude, gps->altitude, gps->speed);
        break;
    case GPS_UNKNOWN:
        /* print unknown statements */
        ESP_LOGW(TAG, "Unknown statement:%s", (char *)event_data);
        break;
    default:
        break;
    }
}

int ATL_SendGPSdata(char* data, int len)            
{
    if(len == (int)0xFFFF)len = strlen((const char*)data);
    const int txBytes = uart_write_bytes(UART_NUM_2, data, len);    
    return txBytes;
}

void Disable_nmea_string(char GGA, char GLL, char GSA, char GSV, char RMC, char VTG, char ZDA,char GST,char GNTXT)
{
    char gps_data[20]={0};
    if(GGA == 1)
    {
       //printf("\r\n********* GGA %d*********\r\n", GGA);
        //              F1                  D9                  06                  01                  03                  00                  F0                  00                  01                  FB                   10 Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x00, gps_data[8] = 0x01, gps_data[9] = 0xFB, gps_data[10] = 0x10; 
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* GGA %d*********\r\n", GGA);
        //              F1                  D9                  06                  01                  03                  00                  F0                  00                  00                  FA                   0F Disable 
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x00, gps_data[8] = 0x00, gps_data[9] = 0xFA, gps_data[10] = 0x0F;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(GLL == 1)
    {
       //printf("\r\n********* GLL %d*********\r\n", GLL);
        //              F1                  D9                  06                  01                  03                  00                  F0                  01                  01                  FC                  12 Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x01, gps_data[8] = 0x01, gps_data[9] = 0xFC, gps_data[10] = 0x12;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* GLL %d*********\r\n", GLL);
        //              F1                  D9                  06                  01                  03                  00                  F0                  01                  00                  FB                  11 Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x01, gps_data[8] = 0x00, gps_data[9] = 0xFB, gps_data[10] = 0x11;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(GSA == 1)
    {
       //printf("\r\n********* GSA %d*********\r\n", GSA);
        //              F1                  D9                  06                  01                  03                  00                  F0                  02                  01                  FD                  14 Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x02, gps_data[8] = 0x01, gps_data[9] = 0xFD, gps_data[10] = 0x14;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* GSA %d*********\r\n", GSA);
        //              F1                  D9                  06                  01                  03                  00                  F0                  02                  00                  FC                  13 Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x02, gps_data[8] = 0x00, gps_data[9] = 0xFC, gps_data[10] = 0x13;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(GSV == 1)
    {
       //printf("\r\n********* GSV %d*********\r\n", GSV);
        //              F1                  D9                  06                  01                  03                  00                  F0                  04                  01                  FF                  18 Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x04, gps_data[8] = 0x01, gps_data[9] = 0xFF, gps_data[10] = 0x18;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* GSV %d*********\r\n", GSV);
        //              F1                  D9                  06                  01                  03                  00                  F0                  04                  00                  FE                  17 Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x04, gps_data[8] = 0x00, gps_data[9] = 0xFE, gps_data[10] = 0x17;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(RMC == 1)
    {
       //printf("\r\n********* RMC %d*********\r\n", RMC);
        //              F1                  D9                  06                  01                  03                  00                  F0                  05                  01                  00                   1A Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x05, gps_data[8] = 0x01, gps_data[9] = 0x00, gps_data[10] = 0x1A;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* RMC %d*********\r\n", RMC);
        //              F1                  D9                  06                  01                  03                  00                  F0                  05                  00                  FF                  19 Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x05, gps_data[8] = 0x00, gps_data[9] = 0xFF, gps_data[10] = 0x19;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(VTG == 1)
    {
       //printf("\r\n********* VTG %d*********\r\n", VTG);
        //              F1                  D9                  06                  01                  03                  00                  F0                  06                  01                  01                  1C Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x06, gps_data[8] = 0x01, gps_data[9] = 0x01, gps_data[10] = 0x1C;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* VTG %d*********\r\n", VTG);
        //              F1                  D9                  06                  01                  03                  00                  F0                  06                  00                  00                  1B Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x06, gps_data[8] = 0x00, gps_data[9] = 0x00, gps_data[10] = 0x1B;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(ZDA == 1)
    {
       //printf("\r\n********* ZDA %d*********\r\n", ZDA);
        //              F1                  D9                  06                  01                  03                  00                  F0                  07                  01                  02                  1E Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x07, gps_data[8] = 0x01, gps_data[9] = 0x02, gps_data[10] = 0x1E;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* ZDA %d*********\r\n", ZDA);
        //              F1                  D9                  06                  01                  03                  00                  F0                  07                  00                  01                  1D Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x07, gps_data[8] = 0x00, gps_data[9] = 0x01, gps_data[10] = 0x1D;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(GST == 1)
    {
       //printf("\r\n********* GST %d*********\r\n", GST);
        //              F1                  D9                  06                  01                  03                  00                  F0                  08                  01                  03                  20 Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x08, gps_data[8] = 0x01, gps_data[9] = 0x03, gps_data[10] = 0x20;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* GST %d*********\r\n", GST);
        //              F1                  D9                  06                  01                  03                  00                  F0                  08                  00                  02                  1F Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x08, gps_data[8] = 0x00, gps_data[9] = 0x02, gps_data[10] = 0x1F;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
     vTaskDelay(1000 / portTICK_PERIOD_MS);
    if(GNTXT == 1)
    {
       //printf("\r\n********* GNTXT %d*********\r\n", GNTXT);
        //              F1                  D9                  06                  01                  03                  00                  F0                  20                  01                  1B                  50 Enable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x20, gps_data[8] = 0x01, gps_data[9] = 0x1B, gps_data[10] = 0x50;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    else
    {
       //printf("\r\n********* GNTXT %d*********\r\n", GNTXT);
        //              F1                  D9                  06                  01                  03                  00                  F0                  20                  00                  1A                  4F Disable
        gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x01, gps_data[4] = 0x03, gps_data[5] = 0x00, gps_data[6] = 0xF0, gps_data[7] = 0x20, gps_data[8] = 0x00, gps_data[9] = 0x1A, gps_data[10] = 0x4F;
        ATL_SendGPSdata((char *)gps_data, 11);
    }
    gps_data[0] = 0xF1, gps_data[1] = 0xD9, gps_data[2] = 0x06, gps_data[3] = 0x09, gps_data[4] = 0x08, gps_data[5] = 0x00, gps_data[6] = 0x00, gps_data[7] = 0x00, gps_data[8] = 0x00, gps_data[9] = 0x00, gps_data[10] = 0x2F, gps_data[11] = 0x00, gps_data[12] = 0x00, gps_data[13] = 0x00,gps_data[14] = 0x46, gps_data[15] = 0x07;
    ATL_SendGPSdata((char *)gps_data, 16);

}

int ms_cnt = 0;
char flag = 0;

bool IRAM_ATTR ATL_sysTick_timer_isr(void *args)
{
    
    ms_cnt++;
    if(ms_cnt >= 1000)                         
    {
       ms_cnt = 0;
       flag = 1;

    }
    return pdFALSE; // return whether we need to yield at the end of ISR
}

void ATL_create_sysTick_timer()
{
    int group=0,timer=0,timer_interval_msec=1;
    bool auto_reload=true;
    timer_config_t config = {
        .divider = TIMER_DIVIDER,
        .counter_dir = TIMER_COUNT_UP,
        .counter_en = TIMER_PAUSE,
        .alarm_en = TIMER_ALARM_EN,
        .auto_reload = auto_reload,
    }; 
    timer_init(group, timer, &config);
    timer_set_counter_value(group, timer, 0);
    timer_set_alarm_value(group, timer, timer_interval_msec * (TIMER_SCALE/1000));
    timer_enable_intr(group, timer);
    timer_isr_callback_add(group, timer, ATL_sysTick_timer_isr, 0, 0);
    timer_start(group, timer);
}

void task1()
{
    while (1)
    {
        write_data();   
        vTaskDelay(100/portTICK_PERIOD_MS);       //printf("function %s\n",__func__);
    }
    
}

void task2()
{
    while (1)
    {
        write_data();  
        vTaskDelay(100/portTICK_PERIOD_MS);       //printf("function %s\n",__func__);
    }
    
}

void app_main(void)
{
    /* NMEA parser configuration */
    memory_mutex = xSemaphoreCreateMutex();
    xTaskCreate(task1, "task1",4096,0,5,NULL);
    xTaskCreate(task2, "task2",4096,0,5,NULL);

    gpio_pad_select_gpio(GPIO_NUM_10);gpio_set_direction(GPIO_NUM_10, GPIO_MODE_OUTPUT);
    gpio_set_level(GPIO_NUM_10, 1);
    ATL_create_sysTick_timer();
    nmea_parser_config_t config = NMEA_PARSER_CONFIG_DEFAULT();
    /* init NMEA parser library */
    nmea_parser_handle_t nmea_hdl = nmea_parser_init(&config);
    /* register event handler for NMEA parser library */
    nmea_parser_add_handler(nmea_hdl, gps_event_handler, NULL);

    while (1)
    {
       if(flag == 1)
       {
        static char ch = 0;
        if(ch == 1)
        {
            CPU_STATUS_ON;
        }
        else
        {
            CPU_STATUS_OFF;
        }
        flag = 0;
       }
       vTaskDelay(10/portTICK_PERIOD_MS);
    }
    
}
